package de.intevation.flys.themes;

import javax.xml.xpath.XPathConstants;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.intevation.artifacts.common.utils.XMLUtils;


/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class ThemeFactory {

    private static Logger logger = Logger.getLogger(ThemeFactory.class);

    private ThemeFactory() {
    }


    /**
     * Creates a new theme from <i>config</i>.
     *
     * @param themeCfg The theme config document that is used to fetch parent
     * themes.
     * @param config The theme config node.
     *
     * @return a new theme.
     */
    public static Theme createTheme(Document themeCfg, Node config) {
        String name = getName(config);
        String desc = getDescription(config);

        logger.debug("Create new theme: " + name);

        Theme theme = new DefaultTheme(name, desc);

        parseInherits(themeCfg, config, theme);
        parseFields(config, theme);
        parseAttrs(config, theme);

        return theme;
    }


    protected static String getName(Node config) {
        return (String) XMLUtils.xpath(config, "@name", XPathConstants.STRING);
    }


    protected static String getDescription(Node config) {
        return (String) XMLUtils.xpath(config, "@desc", XPathConstants.STRING);
    }


    protected static void parseInherits(Document themeCfg, Node cfg, Theme t) {
        logger.debug("ThemeFactory.parseInherits");

        NodeList inherits = (NodeList) XMLUtils.xpath(
            cfg, "inherits/inherit", XPathConstants.NODESET);

        int num = inherits != null ? inherits.getLength() : 0;

        if (num == 0) {
            logger.debug("Theme does not inherit from other themes.");
            return;
        }

        logger.debug("Theme inherits from " + num + " other themes.");

        for (int i = 0; i < num; i++) {
            Node inherit = inherits.item(i);
            String from  = (String) XMLUtils.xpath(
                inherit, "@from", XPathConstants.STRING);

            Node tmp = getThemeNode(themeCfg, from);

            parseInherits(themeCfg, tmp, t);
            parseFields(tmp, t);
        }
    }


    protected static Node getThemeNode(Document themeCfg, String name) {
        if (name == null) {
            logger.warn("Cannot search theme config without name!");
            return null;
        }

        logger.debug("Search for theme: " + name);

        String xpath = "/themes/theme[@name='" + name + "']";

        return (Node) XMLUtils.xpath(themeCfg, xpath, XPathConstants.NODE);
    }


    protected static void parseFields(Node config, Theme theme) {
        if (config == null || theme == null) {
            logger.warn("Parsing fields without node or theme is senseless!");
            return;
        }

        NodeList fields = (NodeList) XMLUtils.xpath(
            config, "fields/field", XPathConstants.NODESET);

        int num = fields != null ? fields.getLength() : 0;

        logger.debug("Found " + num + " own fields in this theme.");

        if (num == 0) {
            logger.debug("Theme has no own fields.");
            return;
        }

        for (int i = 0; i < num; i++) {
            Node field = fields.item(i);

            addField(theme, field);
        }
    }


    protected static void addField(Theme theme, Node field) {
        String name = (String) XMLUtils.xpath(
            field, "@name", XPathConstants.STRING);

        logger.debug("Add field: " + name);

        NamedNodeMap attrs = field.getAttributes();

        int num = attrs != null ? attrs.getLength() : 0;

        if (num == 0) {
            logger.warn("This field has no attributes.");
            return;
        }

        ThemeField theField = new DefaultThemeField(name);

        for (int i = 0; i < num; i++) {
            Node attr    = attrs.item(i);

            String key   = attr.getNodeName();
            String value = attr.getNodeValue();

            theField.setAttribute(key, value);
        }

        theme.addField(name, theField);
    }


    protected static void parseAttrs(Node config, Theme theme) {
        NamedNodeMap attrs = config.getAttributes();

        int num = attrs != null ? attrs.getLength() : 0;

        if (num == 0) {
            logger.debug("Theme has no attributes set.");
            return;
        }

        for (int i = 0; i < num; i++) {
            Node attr = attrs.item(i);

            String name  = attr.getNodeName();
            String value = attr.getNodeValue();

            theme.addAttribute(name, value);
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
